package cn.com.qmth.examcloud.tool.service.export_student_photo;

import cn.com.qmth.examcloud.tool.config.SysProperty;
import cn.com.qmth.examcloud.tool.entity.TaskEntity;
import cn.com.qmth.examcloud.tool.service.CommonService;
import cn.com.qmth.examcloud.tool.service.TaskService;
import cn.com.qmth.examcloud.tool.service.export_student_photo.vo.StudentVO;
import cn.com.qmth.examcloud.tool.utils.FileHelper;
import cn.com.qmth.examcloud.tool.utils.FssHelper;
import cn.com.qmth.examcloud.tool.utils.HttpHelper;
import cn.com.qmth.examcloud.tool.utils.JsonMapper;
import cn.com.qmth.examcloud.tool.vo.PageInfo;
import cn.com.qmth.examcloud.tool.vo.user.User;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class ExportStudentPhotoTask implements TaskService {

    private final static Logger log = LoggerFactory.getLogger(ExportStudentPhotoTask.class);

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private CommonService commonService;

    @Override
    public void start(TaskEntity task) {
        // this.execute(loginUser, task);
    }

    public void execute(User loginUser, String exportDir) {
        // List<StudentVO> hasPhotoStudents = this.loadStudentFromUrl(loginUser);
        List<StudentVO> hasPhotoStudents = this.loadStudentFromExcel(exportDir + "/students.xlsx");
        this.downloadStudents(hasPhotoStudents, exportDir, false);
    }

    private List<StudentVO> loadStudentFromUrl(User loginUser) {
        Map<String, String> headers = new HashMap<>();
        headers.put("key", loginUser.getKey());
        headers.put("token", loginUser.getToken());

        JsonMapper jsonMapper = new JsonMapper();
        long pageNo = 0, pageSize = 100, finishedCount = 0, noPhotoCount = 0;

        final String url = loginUser.getServerUrl() + "/api/ecs_core/student/studentPage/%s/%s?rootOrgId=%s&hasPhoto=TRUE";

        List<StudentVO> hasPhotoStudents = new ArrayList<>();
        while (true) {
            String json = HttpHelper.get(String.format(url, pageNo, pageSize, loginUser.getRootOrgId()), headers, null);
            PageInfo<StudentVO> page = jsonMapper.parseJson(json, new TypeReference<PageInfo<StudentVO>>() {
            });
            if (page == null || CollectionUtils.isEmpty(page.getList())) {
                break;
            }

            for (StudentVO student : page.getList()) {
                if (StringUtils.isEmpty(student.getPhotoPath())) {
                    noPhotoCount++;
                    continue;
                }

                hasPhotoStudents.add(student);
            }

            pageNo++;
            long total = page.getTotal();
            finishedCount += page.getList().size();
            float finishedRate = finishedCount * 100f / total;
            log.info("总数:{} 已获取学生数:{} 无底照学生数:{} pageNo:{} 进度:{}%", total, finishedCount, noPhotoCount, pageNo, finishedRate);
        }

        return hasPhotoStudents;
    }

    private List<StudentVO> loadStudentFromExcel(String dataFilePath) {
        /*
            select stu.id, stu.root_org_id 学校ID, stu.name 学生姓名, stu.identity_number 身份证号,
            (select GROUP_CONCAT(DISTINCT sc.student_code) from ec_b_student_code sc where sc.student_id = stu.id)学号,stu.photo_path 照片
            from ec_b_student stu where stu.root_org_id = 0 and stu.photo_path is not null;
        */
        List<StudentVO> list = new ArrayList<>();
        try {
            EasyExcel.read(dataFilePath, StudentVO.class, new AnalysisEventListener<StudentVO>() {
                        @Override
                        public void invoke(StudentVO data, AnalysisContext analysisContext) {
                            list.add(data);
                        }

                        @Override
                        public void doAfterAllAnalysed(AnalysisContext analysisContext) {
                            // ignore
                        }
                    }
            ).sheet().doRead();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("Excel内容解析错误，请使用标准模板！");
        }

        int noPhotoCount = 0;
        List<StudentVO> hasPhotoStudents = new ArrayList<>();
        for (StudentVO student : list) {
            if (StringUtils.isEmpty(student.getPhotoPath())) {
                noPhotoCount++;
                continue;
            }

            if (StringUtils.isNotBlank(student.getStudentCodesStr())) {
                student.setStudentCodeList(Lists.newArrayList(student.getStudentCodesStr().split(",")));
            }
            hasPhotoStudents.add(student);
        }

        log.info("总数:{} 有底照学生数:{} 无底照学生数:{}", list.size(), hasPhotoStudents.size(), noPhotoCount);
        return hasPhotoStudents;
    }

    private void downloadStudents(List<StudentVO> students, String exportDir, boolean fileNameByStudentCode) {
        for (StudentVO student : students) {
            String photoSuffix = FileHelper.getFileSuffix(student.getPhotoPath());
            if (fileNameByStudentCode) {
                if (CollectionUtils.isNotEmpty(student.getStudentCodeList())) {
                    if (student.getStudentCodeList().size() > 1) {
                        log.warn("identityNumber:{} 多个学号！{}", student.getIdentityNumber(), student.getStudentCodesStr());
                    }
                    for (String studentCode : student.getStudentCodeList()) {
                        String savePhotoPath = String.format("%s/%s/%s%s", exportDir, student.getRootOrgId(), studentCode, photoSuffix);
                        this.downloadPhoto(student.getPhotoPath(), savePhotoPath);
                    }
                } else {
                    log.warn("identityNumber:{} 无学号！", student.getIdentityNumber());
                    String savePhotoPath = String.format("%s/%s/%s%s", exportDir, student.getRootOrgId(), student.getIdentityNumber(), photoSuffix);
                    this.downloadPhoto(student.getPhotoPath(), savePhotoPath);
                }
            } else {
                String savePhotoPath = String.format("%s/%s/%s%s", exportDir, student.getRootOrgId(), student.getIdentityNumber(), photoSuffix);
                this.downloadPhoto(student.getPhotoPath(), savePhotoPath);
            }
        }
    }

    private void downloadPhoto(String photoUrl, String photoPath) {
        String urlPrefix = "https://ecs-test-static.qmth.com.cn";
        photoUrl = FssHelper.finalFileUrl(photoUrl, urlPrefix);

        if (new File(photoPath).exists()) {
            log.info("【已下载】 {} {}", photoPath, photoUrl);
            return;
        }

        try {
            FileHelper.saveImageToFile(photoUrl, photoPath);
        } catch (Exception e) {
            log.error("【错误】 {} err:{}", photoPath, e.getMessage());
        }
    }

}
